@@ -2,6 +2,8 @@
 preserve
 [clinic start generated code]*/
 
+#include "pycore_modsupport.h"    // _PyArg_BadArgument()
+
 PyDoc_STRVAR(_tkinter_tkapp_eval__doc__,
 "eval($self, script, /)\n"
 "--\n"
@@ -14,14 +16,25 @@ static PyObject *
 _tkinter_tkapp_eval_impl(TkappObject *self, const char *script);
 
 static PyObject *
-_tkinter_tkapp_eval(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_eval(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *script;
 
-    if (!PyArg_Parse(arg, "s:eval", &script))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("eval", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_eval_impl(self, script);
+    }
+    Py_ssize_t script_length;
+    script = PyUnicode_AsUTF8AndSize(arg, &script_length);
+    if (script == NULL) {
+        goto exit;
+    }
+    if (strlen(script) != (size_t)script_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_eval_impl((TkappObject *)self, script);
 
 exit:
     return return_value;
@@ -39,14 +52,25 @@ static PyObject *
 _tkinter_tkapp_evalfile_impl(TkappObject *self, const char *fileName);
 
 static PyObject *
-_tkinter_tkapp_evalfile(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_evalfile(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *fileName;
 
-    if (!PyArg_Parse(arg, "s:evalfile", &fileName))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("evalfile", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_evalfile_impl(self, fileName);
+    }
+    Py_ssize_t fileName_length;
+    fileName = PyUnicode_AsUTF8AndSize(arg, &fileName_length);
+    if (fileName == NULL) {
+        goto exit;
+    }
+    if (strlen(fileName) != (size_t)fileName_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_evalfile_impl((TkappObject *)self, fileName);
 
 exit:
     return return_value;
@@ -64,39 +88,61 @@ static PyObject *
 _tkinter_tkapp_record_impl(TkappObject *self, const char *script);
 
 static PyObject *
-_tkinter_tkapp_record(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_record(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *script;
 
-    if (!PyArg_Parse(arg, "s:record", &script))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("record", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_record_impl(self, script);
+    }
+    Py_ssize_t script_length;
+    script = PyUnicode_AsUTF8AndSize(arg, &script_length);
+    if (script == NULL) {
+        goto exit;
+    }
+    if (strlen(script) != (size_t)script_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_record_impl((TkappObject *)self, script);
 
 exit:
     return return_value;
 }
 
-PyDoc_STRVAR(_tkinter_tkapp_adderrinfo__doc__,
-"adderrinfo($self, msg, /)\n"
+PyDoc_STRVAR(_tkinter_tkapp_adderrorinfo__doc__,
+"adderrorinfo($self, msg, /)\n"
 "--\n"
 "\n");
 
-#define _TKINTER_TKAPP_ADDERRINFO_METHODDEF    \
-    {"adderrinfo", (PyCFunction)_tkinter_tkapp_adderrinfo, METH_O, _tkinter_tkapp_adderrinfo__doc__},
+#define _TKINTER_TKAPP_ADDERRORINFO_METHODDEF    \
+    {"adderrorinfo", (PyCFunction)_tkinter_tkapp_adderrorinfo, METH_O, _tkinter_tkapp_adderrorinfo__doc__},
 
 static PyObject *
-_tkinter_tkapp_adderrinfo_impl(TkappObject *self, const char *msg);
+_tkinter_tkapp_adderrorinfo_impl(TkappObject *self, const char *msg);
 
 static PyObject *
-_tkinter_tkapp_adderrinfo(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_adderrorinfo(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *msg;
 
-    if (!PyArg_Parse(arg, "s:adderrinfo", &msg))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("adderrorinfo", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_adderrinfo_impl(self, msg);
+    }
+    Py_ssize_t msg_length;
+    msg = PyUnicode_AsUTF8AndSize(arg, &msg_length);
+    if (msg == NULL) {
+        goto exit;
+    }
+    if (strlen(msg) != (size_t)msg_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_adderrorinfo_impl((TkappObject *)self, msg);
 
 exit:
     return return_value;
@@ -110,6 +156,19 @@ PyDoc_STRVAR(_tkinter_tkapp_getint__doc__,
 #define _TKINTER_TKAPP_GETINT_METHODDEF    \
     {"getint", (PyCFunction)_tkinter_tkapp_getint, METH_O, _tkinter_tkapp_getint__doc__},
 
+static PyObject *
+_tkinter_tkapp_getint_impl(TkappObject *self, PyObject *arg);
+
+static PyObject *
+_tkinter_tkapp_getint(PyObject *self, PyObject *arg)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _tkinter_tkapp_getint_impl((TkappObject *)self, arg);
+
+    return return_value;
+}
+
 PyDoc_STRVAR(_tkinter_tkapp_getdouble__doc__,
 "getdouble($self, arg, /)\n"
 "--\n"
@@ -118,6 +177,19 @@ PyDoc_STRVAR(_tkinter_tkapp_getdouble__doc__,
 #define _TKINTER_TKAPP_GETDOUBLE_METHODDEF    \
     {"getdouble", (PyCFunction)_tkinter_tkapp_getdouble, METH_O, _tkinter_tkapp_getdouble__doc__},
 
+static PyObject *
+_tkinter_tkapp_getdouble_impl(TkappObject *self, PyObject *arg);
+
+static PyObject *
+_tkinter_tkapp_getdouble(PyObject *self, PyObject *arg)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _tkinter_tkapp_getdouble_impl((TkappObject *)self, arg);
+
+    return return_value;
+}
+
 PyDoc_STRVAR(_tkinter_tkapp_getboolean__doc__,
 "getboolean($self, arg, /)\n"
 "--\n"
@@ -126,6 +198,19 @@ PyDoc_STRVAR(_tkinter_tkapp_getboolean__doc__,
 #define _TKINTER_TKAPP_GETBOOLEAN_METHODDEF    \
     {"getboolean", (PyCFunction)_tkinter_tkapp_getboolean, METH_O, _tkinter_tkapp_getboolean__doc__},
 
+static PyObject *
+_tkinter_tkapp_getboolean_impl(TkappObject *self, PyObject *arg);
+
+static PyObject *
+_tkinter_tkapp_getboolean(PyObject *self, PyObject *arg)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _tkinter_tkapp_getboolean_impl((TkappObject *)self, arg);
+
+    return return_value;
+}
+
 PyDoc_STRVAR(_tkinter_tkapp_exprstring__doc__,
 "exprstring($self, s, /)\n"
 "--\n"
@@ -138,14 +223,25 @@ static PyObject *
 _tkinter_tkapp_exprstring_impl(TkappObject *self, const char *s);
 
 static PyObject *
-_tkinter_tkapp_exprstring(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_exprstring(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *s;
 
-    if (!PyArg_Parse(arg, "s:exprstring", &s))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("exprstring", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_exprstring_impl(self, s);
+    }
+    Py_ssize_t s_length;
+    s = PyUnicode_AsUTF8AndSize(arg, &s_length);
+    if (s == NULL) {
+        goto exit;
+    }
+    if (strlen(s) != (size_t)s_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_exprstring_impl((TkappObject *)self, s);
 
 exit:
     return return_value;
@@ -163,14 +259,25 @@ static PyObject *
 _tkinter_tkapp_exprlong_impl(TkappObject *self, const char *s);
 
 static PyObject *
-_tkinter_tkapp_exprlong(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_exprlong(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *s;
 
-    if (!PyArg_Parse(arg, "s:exprlong", &s))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("exprlong", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_exprlong_impl(self, s);
+    }
+    Py_ssize_t s_length;
+    s = PyUnicode_AsUTF8AndSize(arg, &s_length);
+    if (s == NULL) {
+        goto exit;
+    }
+    if (strlen(s) != (size_t)s_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_exprlong_impl((TkappObject *)self, s);
 
 exit:
     return return_value;
@@ -188,14 +295,25 @@ static PyObject *
 _tkinter_tkapp_exprdouble_impl(TkappObject *self, const char *s);
 
 static PyObject *
-_tkinter_tkapp_exprdouble(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_exprdouble(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *s;
 
-    if (!PyArg_Parse(arg, "s:exprdouble", &s))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("exprdouble", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_exprdouble_impl(self, s);
+    }
+    Py_ssize_t s_length;
+    s = PyUnicode_AsUTF8AndSize(arg, &s_length);
+    if (s == NULL) {
+        goto exit;
+    }
+    if (strlen(s) != (size_t)s_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_exprdouble_impl((TkappObject *)self, s);
 
 exit:
     return return_value;
@@ -213,14 +331,25 @@ static PyObject *
 _tkinter_tkapp_exprboolean_impl(TkappObject *self, const char *s);
 
 static PyObject *
-_tkinter_tkapp_exprboolean(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_exprboolean(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *s;
 
-    if (!PyArg_Parse(arg, "s:exprboolean", &s))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("exprboolean", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_exprboolean_impl(self, s);
+    }
+    Py_ssize_t s_length;
+    s = PyUnicode_AsUTF8AndSize(arg, &s_length);
+    if (s == NULL) {
+        goto exit;
+    }
+    if (strlen(s) != (size_t)s_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_exprboolean_impl((TkappObject *)self, s);
 
 exit:
     return return_value;
@@ -234,13 +363,18 @@ PyDoc_STRVAR(_tkinter_tkapp_splitlist__doc__,
 #define _TKINTER_TKAPP_SPLITLIST_METHODDEF    \
     {"splitlist", (PyCFunction)_tkinter_tkapp_splitlist, METH_O, _tkinter_tkapp_splitlist__doc__},
 
-PyDoc_STRVAR(_tkinter_tkapp_split__doc__,
-"split($self, arg, /)\n"
-"--\n"
-"\n");
+static PyObject *
+_tkinter_tkapp_splitlist_impl(TkappObject *self, PyObject *arg);
 
-#define _TKINTER_TKAPP_SPLIT_METHODDEF    \
-    {"split", (PyCFunction)_tkinter_tkapp_split, METH_O, _tkinter_tkapp_split__doc__},
+static PyObject *
+_tkinter_tkapp_splitlist(PyObject *self, PyObject *arg)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _tkinter_tkapp_splitlist_impl((TkappObject *)self, arg);
+
+    return return_value;
+}
 
 PyDoc_STRVAR(_tkinter_tkapp_createcommand__doc__,
 "createcommand($self, name, func, /)\n"
@@ -248,23 +382,37 @@ PyDoc_STRVAR(_tkinter_tkapp_createcommand__doc__,
 "\n");
 
 #define _TKINTER_TKAPP_CREATECOMMAND_METHODDEF    \
-    {"createcommand", (PyCFunction)_tkinter_tkapp_createcommand, METH_VARARGS, _tkinter_tkapp_createcommand__doc__},
+    {"createcommand", _PyCFunction_CAST(_tkinter_tkapp_createcommand), METH_FASTCALL, _tkinter_tkapp_createcommand__doc__},
 
 static PyObject *
 _tkinter_tkapp_createcommand_impl(TkappObject *self, const char *name,
                                   PyObject *func);
 
 static PyObject *
-_tkinter_tkapp_createcommand(TkappObject *self, PyObject *args)
+_tkinter_tkapp_createcommand(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     const char *name;
     PyObject *func;
 
-    if (!PyArg_ParseTuple(args, "sO:createcommand",
-        &name, &func))
+    if (!_PyArg_CheckPositional("createcommand", nargs, 2, 2)) {
         goto exit;
-    return_value = _tkinter_tkapp_createcommand_impl(self, name, func);
+    }
+    if (!PyUnicode_Check(args[0])) {
+        _PyArg_BadArgument("createcommand", "argument 1", "str", args[0]);
+        goto exit;
+    }
+    Py_ssize_t name_length;
+    name = PyUnicode_AsUTF8AndSize(args[0], &name_length);
+    if (name == NULL) {
+        goto exit;
+    }
+    if (strlen(name) != (size_t)name_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    func = args[1];
+    return_value = _tkinter_tkapp_createcommand_impl((TkappObject *)self, name, func);
 
 exit:
     return return_value;
@@ -282,14 +430,25 @@ static PyObject *
 _tkinter_tkapp_deletecommand_impl(TkappObject *self, const char *name);
 
 static PyObject *
-_tkinter_tkapp_deletecommand(TkappObject *self, PyObject *arg)
+_tkinter_tkapp_deletecommand(PyObject *self, PyObject *arg)
 {
     PyObject *return_value = NULL;
     const char *name;
 
-    if (!PyArg_Parse(arg, "s:deletecommand", &name))
+    if (!PyUnicode_Check(arg)) {
+        _PyArg_BadArgument("deletecommand", "argument", "str", arg);
         goto exit;
-    return_value = _tkinter_tkapp_deletecommand_impl(self, name);
+    }
+    Py_ssize_t name_length;
+    name = PyUnicode_AsUTF8AndSize(arg, &name_length);
+    if (name == NULL) {
+        goto exit;
+    }
+    if (strlen(name) != (size_t)name_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    return_value = _tkinter_tkapp_deletecommand_impl((TkappObject *)self, name);
 
 exit:
     return return_value;
@@ -303,24 +462,30 @@ PyDoc_STRVAR(_tkinter_tkapp_createfilehandler__doc__,
 "\n");
 
 #define _TKINTER_TKAPP_CREATEFILEHANDLER_METHODDEF    \
-    {"createfilehandler", (PyCFunction)_tkinter_tkapp_createfilehandler, METH_VARARGS, _tkinter_tkapp_createfilehandler__doc__},
+    {"createfilehandler", _PyCFunction_CAST(_tkinter_tkapp_createfilehandler), METH_FASTCALL, _tkinter_tkapp_createfilehandler__doc__},
 
 static PyObject *
 _tkinter_tkapp_createfilehandler_impl(TkappObject *self, PyObject *file,
                                       int mask, PyObject *func);
 
 static PyObject *
-_tkinter_tkapp_createfilehandler(TkappObject *self, PyObject *args)
+_tkinter_tkapp_createfilehandler(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     PyObject *file;
     int mask;
     PyObject *func;
 
-    if (!PyArg_ParseTuple(args, "OiO:createfilehandler",
-        &file, &mask, &func))
+    if (!_PyArg_CheckPositional("createfilehandler", nargs, 3, 3)) {
         goto exit;
-    return_value = _tkinter_tkapp_createfilehandler_impl(self, file, mask, func);
+    }
+    file = args[0];
+    mask = PyLong_AsInt(args[1]);
+    if (mask == -1 && PyErr_Occurred()) {
+        goto exit;
+    }
+    func = args[2];
+    return_value = _tkinter_tkapp_createfilehandler_impl((TkappObject *)self, file, mask, func);
 
 exit:
     return return_value;
@@ -338,6 +503,19 @@ PyDoc_STRVAR(_tkinter_tkapp_deletefilehandler__doc__,
 #define _TKINTER_TKAPP_DELETEFILEHANDLER_METHODDEF    \
     {"deletefilehandler", (PyCFunction)_tkinter_tkapp_deletefilehandler, METH_O, _tkinter_tkapp_deletefilehandler__doc__},
 
+static PyObject *
+_tkinter_tkapp_deletefilehandler_impl(TkappObject *self, PyObject *file);
+
+static PyObject *
+_tkinter_tkapp_deletefilehandler(PyObject *self, PyObject *file)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _tkinter_tkapp_deletefilehandler_impl((TkappObject *)self, file);
+
+    return return_value;
+}
+
 #endif /* defined(HAVE_CREATEFILEHANDLER) */
 
 PyDoc_STRVAR(_tkinter_tktimertoken_deletetimerhandler__doc__,
@@ -352,9 +530,9 @@ static PyObject *
 _tkinter_tktimertoken_deletetimerhandler_impl(TkttObject *self);
 
 static PyObject *
-_tkinter_tktimertoken_deletetimerhandler(TkttObject *self, PyObject *Py_UNUSED(ignored))
+_tkinter_tktimertoken_deletetimerhandler(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _tkinter_tktimertoken_deletetimerhandler_impl(self);
+    return _tkinter_tktimertoken_deletetimerhandler_impl((TkttObject *)self);
 }
 
 PyDoc_STRVAR(_tkinter_tkapp_createtimerhandler__doc__,
@@ -363,23 +541,28 @@ PyDoc_STRVAR(_tkinter_tkapp_createtimerhandler__doc__,
 "\n");
 
 #define _TKINTER_TKAPP_CREATETIMERHANDLER_METHODDEF    \
-    {"createtimerhandler", (PyCFunction)_tkinter_tkapp_createtimerhandler, METH_VARARGS, _tkinter_tkapp_createtimerhandler__doc__},
+    {"createtimerhandler", _PyCFunction_CAST(_tkinter_tkapp_createtimerhandler), METH_FASTCALL, _tkinter_tkapp_createtimerhandler__doc__},
 
 static PyObject *
 _tkinter_tkapp_createtimerhandler_impl(TkappObject *self, int milliseconds,
                                        PyObject *func);
 
 static PyObject *
-_tkinter_tkapp_createtimerhandler(TkappObject *self, PyObject *args)
+_tkinter_tkapp_createtimerhandler(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     int milliseconds;
     PyObject *func;
 
-    if (!PyArg_ParseTuple(args, "iO:createtimerhandler",
-        &milliseconds, &func))
+    if (!_PyArg_CheckPositional("createtimerhandler", nargs, 2, 2)) {
         goto exit;
-    return_value = _tkinter_tkapp_createtimerhandler_impl(self, milliseconds, func);
+    }
+    milliseconds = PyLong_AsInt(args[0]);
+    if (milliseconds == -1 && PyErr_Occurred()) {
+        goto exit;
+    }
+    func = args[1];
+    return_value = _tkinter_tkapp_createtimerhandler_impl((TkappObject *)self, milliseconds, func);
 
 exit:
     return return_value;
@@ -391,21 +574,29 @@ PyDoc_STRVAR(_tkinter_tkapp_mainloop__doc__,
 "\n");
 
 #define _TKINTER_TKAPP_MAINLOOP_METHODDEF    \
-    {"mainloop", (PyCFunction)_tkinter_tkapp_mainloop, METH_VARARGS, _tkinter_tkapp_mainloop__doc__},
+    {"mainloop", _PyCFunction_CAST(_tkinter_tkapp_mainloop), METH_FASTCALL, _tkinter_tkapp_mainloop__doc__},
 
 static PyObject *
 _tkinter_tkapp_mainloop_impl(TkappObject *self, int threshold);
 
 static PyObject *
-_tkinter_tkapp_mainloop(TkappObject *self, PyObject *args)
+_tkinter_tkapp_mainloop(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     int threshold = 0;
 
-    if (!PyArg_ParseTuple(args, "|i:mainloop",
-        &threshold))
+    if (!_PyArg_CheckPositional("mainloop", nargs, 0, 1)) {
         goto exit;
-    return_value = _tkinter_tkapp_mainloop_impl(self, threshold);
+    }
+    if (nargs < 1) {
+        goto skip_optional;
+    }
+    threshold = PyLong_AsInt(args[0]);
+    if (threshold == -1 && PyErr_Occurred()) {
+        goto exit;
+    }
+skip_optional:
+    return_value = _tkinter_tkapp_mainloop_impl((TkappObject *)self, threshold);
 
 exit:
     return return_value;
@@ -417,21 +608,29 @@ PyDoc_STRVAR(_tkinter_tkapp_dooneevent__doc__,
 "\n");
 
 #define _TKINTER_TKAPP_DOONEEVENT_METHODDEF    \
-    {"dooneevent", (PyCFunction)_tkinter_tkapp_dooneevent, METH_VARARGS, _tkinter_tkapp_dooneevent__doc__},
+    {"dooneevent", _PyCFunction_CAST(_tkinter_tkapp_dooneevent), METH_FASTCALL, _tkinter_tkapp_dooneevent__doc__},
 
 static PyObject *
 _tkinter_tkapp_dooneevent_impl(TkappObject *self, int flags);
 
 static PyObject *
-_tkinter_tkapp_dooneevent(TkappObject *self, PyObject *args)
+_tkinter_tkapp_dooneevent(PyObject *self, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     int flags = 0;
 
-    if (!PyArg_ParseTuple(args, "|i:dooneevent",
-        &flags))
+    if (!_PyArg_CheckPositional("dooneevent", nargs, 0, 1)) {
         goto exit;
-    return_value = _tkinter_tkapp_dooneevent_impl(self, flags);
+    }
+    if (nargs < 1) {
+        goto skip_optional;
+    }
+    flags = PyLong_AsInt(args[0]);
+    if (flags == -1 && PyErr_Occurred()) {
+        goto exit;
+    }
+skip_optional:
+    return_value = _tkinter_tkapp_dooneevent_impl((TkappObject *)self, flags);
 
 exit:
     return return_value;
@@ -449,9 +648,9 @@ static PyObject *
 _tkinter_tkapp_quit_impl(TkappObject *self);
 
 static PyObject *
-_tkinter_tkapp_quit(TkappObject *self, PyObject *Py_UNUSED(ignored))
+_tkinter_tkapp_quit(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _tkinter_tkapp_quit_impl(self);
+    return _tkinter_tkapp_quit_impl((TkappObject *)self);
 }
 
 PyDoc_STRVAR(_tkinter_tkapp_interpaddr__doc__,
@@ -466,9 +665,9 @@ static PyObject *
 _tkinter_tkapp_interpaddr_impl(TkappObject *self);
 
 static PyObject *
-_tkinter_tkapp_interpaddr(TkappObject *self, PyObject *Py_UNUSED(ignored))
+_tkinter_tkapp_interpaddr(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _tkinter_tkapp_interpaddr_impl(self);
+    return _tkinter_tkapp_interpaddr_impl((TkappObject *)self);
 }
 
 PyDoc_STRVAR(_tkinter_tkapp_loadtk__doc__,
@@ -483,9 +682,49 @@ static PyObject *
 _tkinter_tkapp_loadtk_impl(TkappObject *self);
 
 static PyObject *
-_tkinter_tkapp_loadtk(TkappObject *self, PyObject *Py_UNUSED(ignored))
+_tkinter_tkapp_loadtk(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _tkinter_tkapp_loadtk_impl(self);
+    return _tkinter_tkapp_loadtk_impl((TkappObject *)self);
+}
+
+PyDoc_STRVAR(_tkinter_tkapp_settrace__doc__,
+"settrace($self, func, /)\n"
+"--\n"
+"\n"
+"Set the tracing function.");
+
+#define _TKINTER_TKAPP_SETTRACE_METHODDEF    \
+    {"settrace", (PyCFunction)_tkinter_tkapp_settrace, METH_O, _tkinter_tkapp_settrace__doc__},
+
+static PyObject *
+_tkinter_tkapp_settrace_impl(TkappObject *self, PyObject *func);
+
+static PyObject *
+_tkinter_tkapp_settrace(PyObject *self, PyObject *func)
+{
+    PyObject *return_value = NULL;
+
+    return_value = _tkinter_tkapp_settrace_impl((TkappObject *)self, func);
+
+    return return_value;
+}
+
+PyDoc_STRVAR(_tkinter_tkapp_gettrace__doc__,
+"gettrace($self, /)\n"
+"--\n"
+"\n"
+"Get the tracing function.");
+
+#define _TKINTER_TKAPP_GETTRACE_METHODDEF    \
+    {"gettrace", (PyCFunction)_tkinter_tkapp_gettrace, METH_NOARGS, _tkinter_tkapp_gettrace__doc__},
+
+static PyObject *
+_tkinter_tkapp_gettrace_impl(TkappObject *self);
+
+static PyObject *
+_tkinter_tkapp_gettrace(PyObject *self, PyObject *Py_UNUSED(ignored))
+{
+    return _tkinter_tkapp_gettrace_impl((TkappObject *)self);
 }
 
 PyDoc_STRVAR(_tkinter_tkapp_willdispatch__doc__,
@@ -500,9 +739,9 @@ static PyObject *
 _tkinter_tkapp_willdispatch_impl(TkappObject *self);
 
 static PyObject *
-_tkinter_tkapp_willdispatch(TkappObject *self, PyObject *Py_UNUSED(ignored))
+_tkinter_tkapp_willdispatch(PyObject *self, PyObject *Py_UNUSED(ignored))
 {
-    return _tkinter_tkapp_willdispatch_impl(self);
+    return _tkinter_tkapp_willdispatch_impl((TkappObject *)self);
 }
 
 PyDoc_STRVAR(_tkinter__flatten__doc__,
@@ -514,8 +753,8 @@ PyDoc_STRVAR(_tkinter__flatten__doc__,
     {"_flatten", (PyCFunction)_tkinter__flatten, METH_O, _tkinter__flatten__doc__},
 
 PyDoc_STRVAR(_tkinter_create__doc__,
-"create($module, screenName=None, baseName=None, className=\'Tk\',\n"
-"       interactive=False, wantobjects=False, wantTk=True, sync=False,\n"
+"create($module, screenName=None, baseName=\'\', className=\'Tk\',\n"
+"       interactive=False, wantobjects=0, wantTk=True, sync=False,\n"
 "       use=None, /)\n"
 "--\n"
 "\n"
@@ -529,7 +768,7 @@ PyDoc_STRVAR(_tkinter_create__doc__,
 "    if not None, then pass -use to wish");
 
 #define _TKINTER_CREATE_METHODDEF    \
-    {"create", (PyCFunction)_tkinter_create, METH_VARARGS, _tkinter_create__doc__},
+    {"create", _PyCFunction_CAST(_tkinter_create), METH_FASTCALL, _tkinter_create__doc__},
 
 static PyObject *
 _tkinter_create_impl(PyObject *module, const char *screenName,
@@ -538,11 +777,11 @@ _tkinter_create_impl(PyObject *module, const char *screenName,
                      const char *use);
 
 static PyObject *
-_tkinter_create(PyObject *module, PyObject *args)
+_tkinter_create(PyObject *module, PyObject *const *args, Py_ssize_t nargs)
 {
     PyObject *return_value = NULL;
     const char *screenName = NULL;
-    const char *baseName = NULL;
+    const char *baseName = "";
     const char *className = "Tk";
     int interactive = 0;
     int wantobjects = 0;
@@ -550,9 +789,112 @@ _tkinter_create(PyObject *module, PyObject *args)
     int sync = 0;
     const char *use = NULL;
 
-    if (!PyArg_ParseTuple(args, "|zssiiiiz:create",
-        &screenName, &baseName, &className, &interactive, &wantobjects, &wantTk, &sync, &use))
+    if (!_PyArg_CheckPositional("create", nargs, 0, 8)) {
         goto exit;
+    }
+    if (nargs < 1) {
+        goto skip_optional;
+    }
+    if (args[0] == Py_None) {
+        screenName = NULL;
+    }
+    else if (PyUnicode_Check(args[0])) {
+        Py_ssize_t screenName_length;
+        screenName = PyUnicode_AsUTF8AndSize(args[0], &screenName_length);
+        if (screenName == NULL) {
+            goto exit;
+        }
+        if (strlen(screenName) != (size_t)screenName_length) {
+            PyErr_SetString(PyExc_ValueError, "embedded null character");
+            goto exit;
+        }
+    }
+    else {
+        _PyArg_BadArgument("create", "argument 1", "str or None", args[0]);
+        goto exit;
+    }
+    if (nargs < 2) {
+        goto skip_optional;
+    }
+    if (!PyUnicode_Check(args[1])) {
+        _PyArg_BadArgument("create", "argument 2", "str", args[1]);
+        goto exit;
+    }
+    Py_ssize_t baseName_length;
+    baseName = PyUnicode_AsUTF8AndSize(args[1], &baseName_length);
+    if (baseName == NULL) {
+        goto exit;
+    }
+    if (strlen(baseName) != (size_t)baseName_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    if (nargs < 3) {
+        goto skip_optional;
+    }
+    if (!PyUnicode_Check(args[2])) {
+        _PyArg_BadArgument("create", "argument 3", "str", args[2]);
+        goto exit;
+    }
+    Py_ssize_t className_length;
+    className = PyUnicode_AsUTF8AndSize(args[2], &className_length);
+    if (className == NULL) {
+        goto exit;
+    }
+    if (strlen(className) != (size_t)className_length) {
+        PyErr_SetString(PyExc_ValueError, "embedded null character");
+        goto exit;
+    }
+    if (nargs < 4) {
+        goto skip_optional;
+    }
+    interactive = PyObject_IsTrue(args[3]);
+    if (interactive < 0) {
+        goto exit;
+    }
+    if (nargs < 5) {
+        goto skip_optional;
+    }
+    wantobjects = PyLong_AsInt(args[4]);
+    if (wantobjects == -1 && PyErr_Occurred()) {
+        goto exit;
+    }
+    if (nargs < 6) {
+        goto skip_optional;
+    }
+    wantTk = PyObject_IsTrue(args[5]);
+    if (wantTk < 0) {
+        goto exit;
+    }
+    if (nargs < 7) {
+        goto skip_optional;
+    }
+    sync = PyObject_IsTrue(args[6]);
+    if (sync < 0) {
+        goto exit;
+    }
+    if (nargs < 8) {
+        goto skip_optional;
+    }
+    if (args[7] == Py_None) {
+        use = NULL;
+    }
+    else if (PyUnicode_Check(args[7])) {
+        Py_ssize_t use_length;
+        use = PyUnicode_AsUTF8AndSize(args[7], &use_length);
+        if (use == NULL) {
+            goto exit;
+        }
+        if (strlen(use) != (size_t)use_length) {
+            PyErr_SetString(PyExc_ValueError, "embedded null character");
+            goto exit;
+        }
+    }
+    else {
+        _PyArg_BadArgument("create", "argument 8", "str or None", args[7]);
+        goto exit;
+    }
+skip_optional:
     return_value = _tkinter_create_impl(module, screenName, baseName, className, interactive, wantobjects, wantTk, sync, use);
 
 exit:
@@ -579,8 +921,10 @@ _tkinter_setbusywaitinterval(PyObject *module, PyObject *arg)
     PyObject *return_value = NULL;
     int new_val;
 
-    if (!PyArg_Parse(arg, "i:setbusywaitinterval", &new_val))
+    new_val = PyLong_AsInt(arg);
+    if (new_val == -1 && PyErr_Occurred()) {
         goto exit;
+    }
     return_value = _tkinter_setbusywaitinterval_impl(module, new_val);
 
 exit:
@@ -606,8 +950,9 @@ _tkinter_getbusywaitinterval(PyObject *module, PyObject *Py_UNUSED(ignored))
     int _return_value;
 
     _return_value = _tkinter_getbusywaitinterval_impl(module);
-    if ((_return_value == -1) && PyErr_Occurred())
+    if ((_return_value == -1) && PyErr_Occurred()) {
         goto exit;
+    }
     return_value = PyLong_FromLong((long)_return_value);
 
 exit:
@@ -621,4 +966,4 @@ exit:
 #ifndef _TKINTER_TKAPP_DELETEFILEHANDLER_METHODDEF
     #define _TKINTER_TKAPP_DELETEFILEHANDLER_METHODDEF
 #endif /* !defined(_TKINTER_TKAPP_DELETEFILEHANDLER_METHODDEF) */
-/*[clinic end generated code: output=f9057c8bf288633d input=a9049054013a1b77]*/
+/*[clinic end generated code: output=052c067aa69237be input=a9049054013a1b77]*/
