@@ -4,17 +4,22 @@
  * recently, it was largely rewritten by Guido van Rossum.
  */
 
+#ifndef Py_BUILD_CORE_BUILTIN
+#  define Py_BUILD_CORE_MODULE 1
+#endif
+
 /* Standard definitions */
 #include "Python.h"
+#include "pycore_pyatomic_ft_wrappers.h"
+#include "pycore_pylifecycle.h"   // _Py_SetLocaleFromEnv()
 
-#include <errno.h>
-#include <signal.h>
-#include <stddef.h>
+#include <errno.h>                // errno
+#include <signal.h>               // SIGWINCH
 #include <stdlib.h>               // free()
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
+#include <string.h>               // strdup()
+#ifdef HAVE_SYS_SELECT_H
+#  include <sys/select.h>         // select()
 #endif
-#include <time.h>
 
 #if defined(HAVE_SETLOCALE)
 /* GNU readline() mistakenly sets the LC_CTYPE locale.
@@ -22,7 +27,7 @@
  * We must save and restore the locale around the rl_initialize() call.
  */
 #define SAVE_LOCALE
-#include <locale.h>
+#  include <locale.h>             // setlocale()
 #endif
 
 #ifdef SAVE_LOCALE
@@ -143,8 +148,19 @@ readline_free(void *m)
 
 static PyModuleDef readlinemodule;
 
-#define readlinestate_global ((readlinestate *)PyModule_GetState(PyState_FindModule(&readlinemodule)))
-
+static inline readlinestate*
+get_hook_module_state(void)
+{
+    PyObject *mod = PyState_FindModule(&readlinemodule);
+    if (mod == NULL){
+        PyErr_Clear();
+        return NULL;
+    }
+    Py_INCREF(mod);
+    readlinestate *state = get_readline_state(mod);
+    Py_DECREF(mod);
+    return state;
+}
 
 /* Convert to/from multibyte C strings */
 
@@ -184,6 +200,7 @@ disable_bracketed_paste(void)
 /* Exported function to send one line to readline's init file parser */
 
 /*[clinic input]
+@critical_section
 readline.parse_and_bind
 
     string: object
@@ -193,8 +210,8 @@ Execute the init line provided in the string argument.
 [clinic start generated code]*/
 
 static PyObject *
-readline_parse_and_bind(PyObject *module, PyObject *string)
-/*[clinic end generated code: output=1a1ede8afb9546c1 input=8a28a00bb4d61eec]*/
+readline_parse_and_bind_impl(PyObject *module, PyObject *string)
+/*[clinic end generated code: output=828d9b6630d434f5 input=cefdc0f9f62f9fcc]*/
 {
     char *copy;
     PyObject *encoded = encode(string);
@@ -218,6 +235,7 @@ readline_parse_and_bind(PyObject *module, PyObject *string)
 /* Exported function to parse a readline init file */
 
 /*[clinic input]
+@critical_section
 readline.read_init_file
 
     filename as filename_obj: object = None
@@ -230,16 +248,27 @@ The default filename is the last filename used.
 
 static PyObject *
 readline_read_init_file_impl(PyObject *module, PyObject *filename_obj)
-/*[clinic end generated code: output=8e059b676142831e input=4c80c473e448139d]*/
+/*[clinic end generated code: output=8e059b676142831e input=62b767adfab6cc15]*/
 {
     PyObject *filename_bytes;
     if (filename_obj != Py_None) {
         if (!PyUnicode_FSConverter(filename_obj, &filename_bytes))
             return NULL;
+        if (PySys_Audit("open", "OCi", filename_obj, 'r', 0) < 0) {
+            return NULL;
+        }
         errno = rl_read_init_file(PyBytes_AS_STRING(filename_bytes));
         Py_DECREF(filename_bytes);
-    } else
+    } else {
+        /* We have the choice to either try to exactly reproduce the
+         * logic to find the filename, ignore it, or provide a dummy value.
+         * In contract to the history file manipulations, there's no
+         * clear default to choose. */
+        if (PySys_Audit("open", "sCi", "<readline_init_file>", 'r', 0) < 0) {
+            return NULL;
+        }
         errno = rl_read_init_file(NULL);
+    }
     if (errno)
         return PyErr_SetFromErrno(PyExc_OSError);
     disable_bracketed_paste();
@@ -249,6 +278,7 @@ readline_read_init_file_impl(PyObject *module, PyObject *filename_obj)
 /* Exported function to load a readline history file */
 
 /*[clinic input]
+@critical_section
 readline.read_history_file
 
     filename as filename_obj: object = None
@@ -261,16 +291,25 @@ The default filename is ~/.history.
 
 static PyObject *
 readline_read_history_file_impl(PyObject *module, PyObject *filename_obj)
-/*[clinic end generated code: output=66a951836fb54fbb input=3d29d755b7e6932e]*/
+/*[clinic end generated code: output=66a951836fb54fbb input=5d86fd7813172a67]*/
 {
     PyObject *filename_bytes;
     if (filename_obj != Py_None) {
         if (!PyUnicode_FSConverter(filename_obj, &filename_bytes))
             return NULL;
+        if (PySys_Audit("open", "OCi", filename_obj, 'r', 0) < 0) {
+            return NULL;
+        }
         errno = read_history(PyBytes_AS_STRING(filename_bytes));
         Py_DECREF(filename_bytes);
-    } else
+    } else {
+        /* Use the documented default filename here,
+         * even though readline expands it different internally. */
+        if (PySys_Audit("open", "sCi", "~/.history", 'r', 0) < 0) {
+            return NULL;
+        }
         errno = read_history(NULL);
+    }
     if (errno)
         return PyErr_SetFromErrno(PyExc_OSError);
     Py_RETURN_NONE;
@@ -281,6 +320,7 @@ static int _history_length = -1; /* do not truncate history by default */
 /* Exported function to save a readline history file */
 
 /*[clinic input]
+@critical_section
 readline.write_history_file
 
     filename as filename_obj: object = None
@@ -293,7 +333,7 @@ The default filename is ~/.history.
 
 static PyObject *
 readline_write_history_file_impl(PyObject *module, PyObject *filename_obj)
-/*[clinic end generated code: output=fbcad13d8ef59ae6 input=28a8e062fe363703]*/
+/*[clinic end generated code: output=fbcad13d8ef59ae6 input=34aaada95120cfaa]*/
 {
     PyObject *filename_bytes;
     const char *filename;
@@ -302,13 +342,22 @@ readline_write_history_file_impl(PyObject *module, PyObject *filename_obj)
         if (!PyUnicode_FSConverter(filename_obj, &filename_bytes))
             return NULL;
         filename = PyBytes_AS_STRING(filename_bytes);
+        if (PySys_Audit("open", "OCi", filename_obj, 'w', 0) < 0) {
+            return NULL;
+        }
     } else {
         filename_bytes = NULL;
         filename = NULL;
+        /* Use the documented default filename here,
+         * even though readline expands it different internally. */
+        if (PySys_Audit("open", "sCi", "~/.history", 'w', 0) < 0) {
+            return NULL;
+        }
     }
     errno = err = write_history(filename);
-    if (!err && _history_length >= 0)
-        history_truncate_file(filename, _history_length);
+    int history_length = FT_ATOMIC_LOAD_INT_RELAXED(_history_length);
+    if (!err && history_length >= 0)
+        history_truncate_file(filename, history_length);
     Py_XDECREF(filename_bytes);
     errno = err;
     if (errno)
@@ -320,6 +369,7 @@ readline_write_history_file_impl(PyObject *module, PyObject *filename_obj)
 /* Exported function to save part of a readline history file */
 
 /*[clinic input]
+@critical_section
 readline.append_history_file
 
     nelements: int
@@ -334,8 +384,14 @@ The default filename is ~/.history.
 static PyObject *
 readline_append_history_file_impl(PyObject *module, int nelements,
                                   PyObject *filename_obj)
-/*[clinic end generated code: output=5df06fc9da56e4e4 input=784b774db3a4b7c5]*/
+/*[clinic end generated code: output=5df06fc9da56e4e4 input=78a6061a8d3a0275]*/
 {
+    if (nelements < 0)
+    {
+        PyErr_SetString(PyExc_ValueError, "nelements must be positive");
+        return NULL;
+    }
+
     PyObject *filename_bytes;
     const char *filename;
     int err;
@@ -343,14 +399,23 @@ readline_append_history_file_impl(PyObject *module, int nelements,
         if (!PyUnicode_FSConverter(filename_obj, &filename_bytes))
             return NULL;
         filename = PyBytes_AS_STRING(filename_bytes);
+        if (PySys_Audit("open", "OCi", filename_obj, 'a', 0) < 0) {
+            return NULL;
+        }
     } else {
         filename_bytes = NULL;
         filename = NULL;
+        /* Use the documented default filename here,
+         * even though readline expands it different internally. */
+        if (PySys_Audit("open", "sCi", "~/.history", 'a', 0) < 0) {
+            return NULL;
+        }
     }
     errno = err = append_history(
         nelements - libedit_append_replace_history_offset, filename);
-    if (!err && _history_length >= 0)
-        history_truncate_file(filename, _history_length);
+    int history_length = FT_ATOMIC_LOAD_INT_RELAXED(_history_length);
+    if (!err && history_length >= 0)
+        history_truncate_file(filename, history_length);
     Py_XDECREF(filename_bytes);
     errno = err;
     if (errno)
@@ -377,7 +442,7 @@ static PyObject *
 readline_set_history_length_impl(PyObject *module, int length)
 /*[clinic end generated code: output=e161a53e45987dc7 input=b8901bf16488b760]*/
 {
-    _history_length = length;
+    FT_ATOMIC_STORE_INT_RELAXED(_history_length, length);
     Py_RETURN_NONE;
 }
 
@@ -393,7 +458,8 @@ static PyObject *
 readline_get_history_length_impl(PyObject *module)
 /*[clinic end generated code: output=83a2eeae35b6d2b9 input=5dce2eeba4327817]*/
 {
-    return PyLong_FromLong(_history_length);
+    int history_length = FT_ATOMIC_LOAD_INT_RELAXED(_history_length);
+    return PyLong_FromLong(history_length);
 }
 
 /* Generic hook function setter */
@@ -417,6 +483,7 @@ set_hook(const char *funcname, PyObject **hook_var, PyObject *function)
 }
 
 /*[clinic input]
+@critical_section
 readline.set_completion_display_matches_hook
 
     function: object = None
@@ -432,16 +499,17 @@ once each time matches need to be displayed.
 static PyObject *
 readline_set_completion_display_matches_hook_impl(PyObject *module,
                                                   PyObject *function)
-/*[clinic end generated code: output=516e5cb8db75a328 input=4f0bfd5ab0179a26]*/
+/*[clinic end generated code: output=516e5cb8db75a328 input=ea4191e4a07d28d3]*/
 {
+    readlinestate *state = get_readline_state(module);
     PyObject *result = set_hook("completion_display_matches_hook",
-                    &readlinestate_global->completion_display_matches_hook,
+                    &state->completion_display_matches_hook,
                     function);
 #ifdef HAVE_RL_COMPLETION_DISPLAY_MATCHES_HOOK
     /* We cannot set this hook globally, since it replaces the
        default completion display. */
     rl_completion_display_matches_hook =
-        readlinestate_global->completion_display_matches_hook ?
+        state->completion_display_matches_hook ?
 #if defined(HAVE_RL_COMPDISP_FUNC_T)
         (rl_compdisp_func_t *)on_completion_display_matches_hook : 0;
 #else
@@ -453,6 +521,7 @@ readline_set_completion_display_matches_hook_impl(PyObject *module,
 }
 
 /*[clinic input]
+@critical_section
 readline.set_startup_hook
 
     function: object = None
@@ -466,9 +535,10 @@ before readline prints the first prompt.
 
 static PyObject *
 readline_set_startup_hook_impl(PyObject *module, PyObject *function)
-/*[clinic end generated code: output=02cd0e0c4fa082ad input=7783b4334b26d16d]*/
+/*[clinic end generated code: output=02cd0e0c4fa082ad input=11fce34992f1125e]*/
 {
-    return set_hook("startup_hook", &readlinestate_global->startup_hook,
+    readlinestate *state = get_readline_state(module);
+    return set_hook("startup_hook", &state->startup_hook,
             function);
 }
 
@@ -477,6 +547,7 @@ readline_set_startup_hook_impl(PyObject *module, PyObject *function)
 /* Set pre-input hook */
 
 /*[clinic input]
+@critical_section
 readline.set_pre_input_hook
 
     function: object = None
@@ -491,9 +562,10 @@ characters.
 
 static PyObject *
 readline_set_pre_input_hook_impl(PyObject *module, PyObject *function)
-/*[clinic end generated code: output=fe1a96505096f464 input=4f3eaeaf7ce1fdbe]*/
+/*[clinic end generated code: output=fe1a96505096f464 input=96d3d5ff4a0c7c28]*/
 {
-    return set_hook("pre_input_hook", &readlinestate_global->pre_input_hook,
+    readlinestate *state = get_readline_state(module);
+    return set_hook("pre_input_hook", &state->pre_input_hook,
             function);
 }
 #endif
@@ -526,7 +598,8 @@ static PyObject *
 readline_get_begidx_impl(PyObject *module)
 /*[clinic end generated code: output=362616ee8ed1b2b1 input=e083b81c8eb4bac3]*/
 {
-    return Py_NewRef(readlinestate_global->begidx);
+    readlinestate *state = get_readline_state(module);
+    return Py_NewRef(state->begidx);
 }
 
 /* Get the ending index for the scope of the tab-completion */
@@ -541,12 +614,14 @@ static PyObject *
 readline_get_endidx_impl(PyObject *module)
 /*[clinic end generated code: output=7f763350b12d7517 input=d4c7e34a625fd770]*/
 {
-    return Py_NewRef(readlinestate_global->endidx);
+    readlinestate *state = get_readline_state(module);
+    return Py_NewRef(state->endidx);
 }
 
 /* Set the tab-completion word-delimiters that readline uses */
 
 /*[clinic input]
+@critical_section
 readline.set_completer_delims
 
     string: object
@@ -556,8 +631,8 @@ Set the word delimiters for completion.
 [clinic start generated code]*/
 
 static PyObject *
-readline_set_completer_delims(PyObject *module, PyObject *string)
-/*[clinic end generated code: output=4305b266106c4f1f input=ae945337ebd01e20]*/
+readline_set_completer_delims_impl(PyObject *module, PyObject *string)
+/*[clinic end generated code: output=017e48e9704a2f64 input=6c87bb1cbed7fcf1]*/
 {
     char *break_chars;
     PyObject *encoded = encode(string);
@@ -586,7 +661,7 @@ readline_set_completer_delims(PyObject *module, PyObject *string)
         return PyErr_NoMemory();
 }
 
-/* _py_free_history_entry: Utility function to free a history entry. */
+/* _py_free_history_entry_lock_held: Utility function to free a history entry. */
 
 #if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0500
 
@@ -596,7 +671,7 @@ readline_set_completer_delims(PyObject *module, PyObject *string)
    takes care of the timestamp. */
 
 static void
-_py_free_history_entry(HIST_ENTRY *entry)
+_py_free_history_entry_lock_held(HIST_ENTRY *entry)
 {
     histdata_t data = free_history_entry(entry);
     free(data);
@@ -607,7 +682,7 @@ _py_free_history_entry(HIST_ENTRY *entry)
 /* No free_history_entry function;  free everything manually. */
 
 static void
-_py_free_history_entry(HIST_ENTRY *entry)
+_py_free_history_entry_lock_held(HIST_ENTRY *entry)
 {
     if (entry->line)
         free((void *)entry->line);
@@ -619,6 +694,7 @@ _py_free_history_entry(HIST_ENTRY *entry)
 #endif
 
 /*[clinic input]
+@critical_section
 readline.remove_history_item
 
     pos as entry_number: int
@@ -629,7 +705,7 @@ Remove history item given by its zero-based position.
 
 static PyObject *
 readline_remove_history_item_impl(PyObject *module, int entry_number)
-/*[clinic end generated code: output=ab114f029208c7e8 input=f248beb720ff1838]*/
+/*[clinic end generated code: output=ab114f029208c7e8 input=847d7cc7e7c25852]*/
 {
     HIST_ENTRY *entry;
 
@@ -646,11 +722,12 @@ readline_remove_history_item_impl(PyObject *module, int entry_number)
         return NULL;
     }
     /* free memory allocated for the history entry */
-    _py_free_history_entry(entry);
+    _py_free_history_entry_lock_held(entry);
     Py_RETURN_NONE;
 }
 
 /*[clinic input]
+@critical_section
 readline.replace_history_item
 
     pos as entry_number: int
@@ -665,7 +742,7 @@ pos is zero-based.
 static PyObject *
 readline_replace_history_item_impl(PyObject *module, int entry_number,
                                    PyObject *line)
-/*[clinic end generated code: output=f8cec2770ca125eb input=368bb66fe5ee5222]*/
+/*[clinic end generated code: output=f8cec2770ca125eb input=b44c8dcdc2dd87fe]*/
 {
     PyObject *encoded;
     HIST_ENTRY *old_entry;
@@ -690,13 +767,14 @@ readline_replace_history_item_impl(PyObject *module, int entry_number,
         return NULL;
     }
     /* free memory allocated for the old history entry */
-    _py_free_history_entry(old_entry);
+    _py_free_history_entry_lock_held(old_entry);
     Py_RETURN_NONE;
 }
 
 /* Add a line to the history buffer */
 
 /*[clinic input]
+@critical_section
 readline.add_history
 
     string: object
@@ -706,8 +784,8 @@ Add an item to the history buffer.
 [clinic start generated code]*/
 
 static PyObject *
-readline_add_history(PyObject *module, PyObject *string)
-/*[clinic end generated code: output=b107b7e8106e803d input=e57c1cf6bc68d7e3]*/
+readline_add_history_impl(PyObject *module, PyObject *string)
+/*[clinic end generated code: output=89047062042ac344 input=faa7053b8612513b]*/
 {
     PyObject *encoded = encode(string);
     if (encoded == NULL) {
@@ -744,6 +822,7 @@ readline_set_auto_history_impl(PyObject *module,
 /* Get the tab-completion word-delimiters that readline uses */
 
 /*[clinic input]
+@critical_section
 readline.get_completer_delims
 
 Get the word delimiters for completion.
@@ -751,7 +830,7 @@ Get the word delimiters for completion.
 
 static PyObject *
 readline_get_completer_delims_impl(PyObject *module)
-/*[clinic end generated code: output=6b060280fa68ef43 input=e36eb14fb8a1f08a]*/
+/*[clinic end generated code: output=6b060280fa68ef43 input=80583cdf8176bcdd]*/
 {
     return decode(rl_completer_word_break_characters);
 }
@@ -759,6 +838,7 @@ readline_get_completer_delims_impl(PyObject *module)
 /* Set the completer function */
 
 /*[clinic input]
+@critical_section
 readline.set_completer
 
     function: object = None
@@ -773,9 +853,10 @@ It should return the next possible completion starting with 'text'.
 
 static PyObject *
 readline_set_completer_impl(PyObject *module, PyObject *function)
-/*[clinic end generated code: output=171a2a60f81d3204 input=51e81e13118eb877]*/
+/*[clinic end generated code: output=171a2a60f81d3204 input=97f539d8d0bfcb95]*/
 {
-    return set_hook("completer", &readlinestate_global->completer, function);
+    readlinestate *state = get_readline_state(module);
+    return set_hook("completer", &state->completer, function);
 }
 
 /*[clinic input]
@@ -788,10 +869,11 @@ static PyObject *
 readline_get_completer_impl(PyObject *module)
 /*[clinic end generated code: output=6e6bbd8226d14475 input=6457522e56d70d13]*/
 {
-    if (readlinestate_global->completer == NULL) {
+    readlinestate *state = get_readline_state(module);
+    if (state->completer == NULL) {
         Py_RETURN_NONE;
     }
-    return Py_NewRef(readlinestate_global->completer);
+    return Py_NewRef(state->completer);
 }
 
 /* Private function to get current length of history.  XXX It may be
@@ -800,7 +882,7 @@ readline_get_completer_impl(PyObject *module)
  * See issue #8065.*/
 
 static int
-_py_get_history_length(void)
+_py_get_history_length_lock_held(void)
 {
     HISTORY_STATE *hist_st = history_get_history_state();
     int length = hist_st->length;
@@ -816,6 +898,7 @@ _py_get_history_length(void)
 /* Exported function to get any element of history */
 
 /*[clinic input]
+@critical_section
 readline.get_history_item
 
     index as idx: int
@@ -826,7 +909,7 @@ Return the current contents of history item at one-based index.
 
 static PyObject *
 readline_get_history_item_impl(PyObject *module, int idx)
-/*[clinic end generated code: output=83d3e53ea5f34b3d input=8adf5c80e6c7ff2b]*/
+/*[clinic end generated code: output=83d3e53ea5f34b3d input=2835b50c7bde705f]*/
 {
     HIST_ENTRY *hist_ent;
 
@@ -835,7 +918,7 @@ readline_get_history_item_impl(PyObject *module, int idx)
          * use 0-based indexes, while readline and newer
          * versions of libedit use 1-based indexes.
          */
-        int length = _py_get_history_length();
+        int length = _py_get_history_length_lock_held();
 
         idx = idx - 1 + libedit_history_start;
 
@@ -859,6 +942,7 @@ readline_get_history_item_impl(PyObject *module, int idx)
 /* Exported function to get current length of history */
 
 /*[clinic input]
+@critical_section
 readline.get_current_history_length
 
 Return the current (not the maximum) length of history.
@@ -866,14 +950,15 @@ Return the current (not the maximum) length of history.
 
 static PyObject *
 readline_get_current_history_length_impl(PyObject *module)
-/*[clinic end generated code: output=436b294f12ba1e3f input=9cb3f431a68d071f]*/
+/*[clinic end generated code: output=436b294f12ba1e3f input=22e9fd0abbc2fd8d]*/
 {
-    return PyLong_FromLong((long)_py_get_history_length());
+    return PyLong_FromLong((long)_py_get_history_length_lock_held());
 }
 
 /* Exported function to read the current line buffer */
 
 /*[clinic input]
+@critical_section
 readline.get_line_buffer
 
 Return the current contents of the line buffer.
@@ -881,7 +966,7 @@ Return the current contents of the line buffer.
 
 static PyObject *
 readline_get_line_buffer_impl(PyObject *module)
-/*[clinic end generated code: output=d22f9025ecad80e4 input=5f5fbc0d12c69412]*/
+/*[clinic end generated code: output=d22f9025ecad80e4 input=8e02e0fe081feece]*/
 {
     return decode(rl_line_buffer);
 }
@@ -891,6 +976,7 @@ readline_get_line_buffer_impl(PyObject *module)
 /* Exported function to clear the current history */
 
 /*[clinic input]
+@critical_section
 readline.clear_history
 
 Clear the current readline history.
@@ -898,7 +984,7 @@ Clear the current readline history.
 
 static PyObject *
 readline_clear_history_impl(PyObject *module)
-/*[clinic end generated code: output=1f2dbb0dfa5d5ebb input=208962c4393f5d16]*/
+/*[clinic end generated code: output=1f2dbb0dfa5d5ebb input=b2c6b11551593053]*/
 {
     clear_history();
     Py_RETURN_NONE;
@@ -909,6 +995,7 @@ readline_clear_history_impl(PyObject *module)
 /* Exported function to insert text into the line buffer */
 
 /*[clinic input]
+@critical_section
 readline.insert_text
 
     string: object
@@ -918,8 +1005,8 @@ Insert text into the line buffer at the cursor position.
 [clinic start generated code]*/
 
 static PyObject *
-readline_insert_text(PyObject *module, PyObject *string)
-/*[clinic end generated code: output=23d792821d320c19 input=bc96c3c848d5ccb5]*/
+readline_insert_text_impl(PyObject *module, PyObject *string)
+/*[clinic end generated code: output=4bf4e176f68750e0 input=2f401f4316df33c2]*/
 {
     PyObject *encoded = encode(string);
     if (encoded == NULL) {
@@ -933,6 +1020,7 @@ readline_insert_text(PyObject *module, PyObject *string)
 /* Redisplay the line buffer */
 
 /*[clinic input]
+@critical_section
 readline.redisplay
 
 Change what's displayed on the screen to reflect contents of the line buffer.
@@ -940,7 +1028,7 @@ Change what's displayed on the screen to reflect contents of the line buffer.
 
 static PyObject *
 readline_redisplay_impl(PyObject *module)
-/*[clinic end generated code: output=a8b9725827c3c34b input=b485151058d75edc]*/
+/*[clinic end generated code: output=a8b9725827c3c34b input=5895fd014615ff58]*/
 {
     rl_redisplay();
     Py_RETURN_NONE;
@@ -1003,7 +1091,7 @@ on_hook(PyObject *func)
         if (r == Py_None)
             result = 0;
         else {
-            result = _PyLong_AsInt(r);
+            result = PyLong_AsInt(r);
             if (result == -1 && PyErr_Occurred())
                 goto error;
         }
@@ -1019,30 +1107,40 @@ on_hook(PyObject *func)
 }
 
 static int
-#if defined(_RL_FUNCTION_TYPEDEF)
+#if defined(_RL_FUNCTION_TYPEDEF) || !defined(Py_RL_STARTUP_HOOK_TAKES_ARGS)
 on_startup_hook(void)
 #else
-on_startup_hook()
+on_startup_hook(const char *Py_UNUSED(text), int Py_UNUSED(state))
 #endif
 {
     int r;
     PyGILState_STATE gilstate = PyGILState_Ensure();
-    r = on_hook(readlinestate_global->startup_hook);
+    readlinestate *state = get_hook_module_state();
+    if (state == NULL) {
+        PyGILState_Release(gilstate);
+        return -1;
+    }
+    r = on_hook(state->startup_hook);
     PyGILState_Release(gilstate);
     return r;
 }
 
 #ifdef HAVE_RL_PRE_INPUT_HOOK
 static int
-#if defined(_RL_FUNCTION_TYPEDEF)
+#if defined(_RL_FUNCTION_TYPEDEF) || !defined(Py_RL_STARTUP_HOOK_TAKES_ARGS)
 on_pre_input_hook(void)
 #else
-on_pre_input_hook()
+on_pre_input_hook(const char *Py_UNUSED(text), int Py_UNUSED(state))
 #endif
 {
     int r;
     PyGILState_STATE gilstate = PyGILState_Ensure();
-    r = on_hook(readlinestate_global->pre_input_hook);
+    readlinestate *state = get_hook_module_state();
+    if (state == NULL) {
+        PyGILState_Release(gilstate);
+        return -1;
+    }
+    r = on_hook(state->pre_input_hook);
     PyGILState_Release(gilstate);
     return r;
 }
@@ -1059,6 +1157,11 @@ on_completion_display_matches_hook(char **matches,
     int i;
     PyObject *sub, *m=NULL, *s=NULL, *r=NULL;
     PyGILState_STATE gilstate = PyGILState_Ensure();
+    readlinestate *state = get_hook_module_state();
+    if (state == NULL) {
+        PyGILState_Release(gilstate);
+        return;
+    }
     m = PyList_New(num_matches);
     if (m == NULL)
         goto error;
@@ -1069,7 +1172,7 @@ on_completion_display_matches_hook(char **matches,
         PyList_SET_ITEM(m, i, s);
     }
     sub = decode(matches[0]);
-    r = PyObject_CallFunction(readlinestate_global->completion_display_matches_hook,
+    r = PyObject_CallFunction(state->completion_display_matches_hook,
                               "NNi", sub, m, max_length);
 
     m=NULL;
@@ -1117,12 +1220,17 @@ static char *
 on_completion(const char *text, int state)
 {
     char *result = NULL;
-    if (readlinestate_global->completer != NULL) {
+    PyGILState_STATE gilstate = PyGILState_Ensure();
+    readlinestate *module_state = get_hook_module_state();
+    if (module_state == NULL) {
+        PyGILState_Release(gilstate);
+        return NULL;
+    }
+    if (module_state->completer != NULL) {
         PyObject *r = NULL, *t;
-        PyGILState_STATE gilstate = PyGILState_Ensure();
         rl_attempted_completion_over = 1;
         t = decode(text);
-        r = PyObject_CallFunction(readlinestate_global->completer, "Ni", t, state);
+        r = PyObject_CallFunction(module_state->completer, "Ni", t, state);
         if (r == NULL)
             goto error;
         if (r == Py_None) {
@@ -1144,6 +1252,7 @@ on_completion(const char *text, int state)
         PyGILState_Release(gilstate);
         return result;
     }
+    PyGILState_Release(gilstate);
     return result;
 }
 
@@ -1159,6 +1268,7 @@ flex_complete(const char *text, int start, int end)
     size_t start_size, end_size;
     wchar_t *s;
     PyGILState_STATE gilstate = PyGILState_Ensure();
+    readlinestate *state = get_hook_module_state();
 #ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
     rl_completion_append_character ='\0';
 #endif
@@ -1186,10 +1296,12 @@ flex_complete(const char *text, int start, int end)
     end = start + (int)end_size;
 
 done:
-    Py_XDECREF(readlinestate_global->begidx);
-    Py_XDECREF(readlinestate_global->endidx);
-    readlinestate_global->begidx = PyLong_FromLong((long) start);
-    readlinestate_global->endidx = PyLong_FromLong((long) end);
+    if (state) {
+        Py_XDECREF(state->begidx);
+        Py_XDECREF(state->endidx);
+        state->begidx = PyLong_FromLong((long) start);
+        state->endidx = PyLong_FromLong((long) end);
+    }
     result = completion_matches((char *)text, *on_completion);
     PyGILState_Release(gilstate);
     return result;
@@ -1234,7 +1346,7 @@ setup_readline(readlinestate *mod_state)
     {
         add_history("2");
         HIST_ENTRY *old_entry = replace_history_entry(1, "X", NULL);
-        _py_free_history_entry(old_entry);
+        _py_free_history_entry_lock_held(old_entry);
         HIST_ENTRY *item = history_get(libedit_history_start);
         if (item && item->line && strcmp(item->line, "X")) {
             libedit_append_replace_history_offset = 0;
@@ -1324,6 +1436,9 @@ rlhandler(char *text)
 static char *
 readline_until_enter_or_signal(const char *prompt, int *signal)
 {
+    // Defined in Parser/myreadline.c
+    extern PyThreadState *_PyOS_ReadlineTState;
+
     char * not_done_reading = "";
     fd_set selectset;
 
@@ -1341,7 +1456,8 @@ readline_until_enter_or_signal(const char *prompt, int *signal)
         int has_input = 0, err = 0;
 
         while (!has_input)
-        {               struct timeval timeout = {0, 100000}; /* 0.1 seconds */
+        {
+            struct timeval timeout = {0, 100000};  // 100 ms (0.1 seconds)
 
             /* [Bug #1552726] Only limit the pause if an input hook has been
                defined.  */
@@ -1431,7 +1547,7 @@ call_readline(FILE *sys_stdin, FILE *sys_stdout, const char *prompt)
     n = strlen(p);
     if (should_auto_add_history && n > 0) {
         const char *line;
-        int length = _py_get_history_length();
+        int length = _py_get_history_length_lock_held();
         if (length > 0) {
             HIST_ENTRY *hist_ent;
             if (using_libedit_emulation) {
@@ -1484,6 +1600,7 @@ static struct PyModuleDef readlinemodule = {
 PyMODINIT_FUNC
 PyInit_readline(void)
 {
+    const char *backend = "readline";
     PyObject *m;
     readlinestate *mod_state;
 
@@ -1491,14 +1608,19 @@ PyInit_readline(void)
         using_libedit_emulation = 1;
     }
 
-    if (using_libedit_emulation)
+    if (using_libedit_emulation) {
         readlinemodule.m_doc = doc_module_le;
+        backend = "editline";
+    }
 
 
     m = PyModule_Create(&readlinemodule);
 
     if (m == NULL)
         return NULL;
+#ifdef Py_GIL_DISABLED
+    PyUnstable_Module_SetGIL(m, Py_MOD_GIL_NOT_USED);
+#endif
 
     if (PyModule_AddIntConstant(m, "_READLINE_VERSION",
                                 RL_READLINE_VERSION) < 0) {
@@ -1514,13 +1636,22 @@ PyInit_readline(void)
         goto error;
     }
 
+    if (PyModule_AddStringConstant(m, "backend", backend) < 0) {
+        goto error;
+    }
+
     mod_state = (readlinestate *) PyModule_GetState(m);
+    if (mod_state == NULL){
+        goto error;
+    }
     PyOS_ReadlineFunctionPointer = call_readline;
     if (setup_readline(mod_state) < 0) {
         PyErr_NoMemory();
         goto error;
     }
-
+    if (PyErr_Occurred()){
+        goto error;
+    }
     return m;
 
 error:
