@@ -12,7 +12,7 @@ conditions that must be applied when parsing C code:
 
 * ...
 
-(see: http://www.open-std.org/jtc1/sc22/wg14/www/docs/n1256.pdf)
+(see: https://www.open-std.org/jtc1/sc22/wg14/www/docs/n1256.pdf)
 
 We have taken advantage of the elements of the C grammar that are used
 only in a few limited contexts, mostly as delimiters.  They allow us to
@@ -42,7 +42,7 @@ separators:
    + (stmt) case:  between expression and stmt
    + (stmt) default:  between "default" and stmt
 * "="
-   + (decl) delaration:  between decl and initializer
+   + (decl) declaration:  between decl and initializer
    + (decl) enumerator:  between identifier and "initializer"
    + (expr) assignment:  between "var" and expr
 
@@ -92,7 +92,7 @@ Here are the cases where we've taken shortcuts or made assumptions:
 * no "inline" type decls in function return types
 * no superfluous parentheses in declarators
 * var decls in for loops are always "simple" (e.g. no inline types)
-* only inline struct/union/enum decls may be anonymouns (without a name)
+* only inline struct/union/enum decls may be anonymous (without a name)
 * no function pointers in function pointer parameters
 * for loop "headers" do not have curly braces (e.g. compound init)
 * syntactically, variable decls do not overlap with stmts/exprs, except
@@ -120,12 +120,12 @@ from ..info import ParsedItem
 from ._info import SourceInfo
 
 
-def parse(srclines):
+def parse(srclines, **srckwargs):
     if isinstance(srclines, str):  # a filename
         raise NotImplementedError
 
     anon_name = anonymous_names()
-    for result in _parse(srclines, anon_name):
+    for result in _parse(srclines, anon_name, **srckwargs):
         yield ParsedItem.from_raw(result)
 
 
@@ -152,17 +152,19 @@ import logging
 _logger = logging.getLogger(__name__)
 
 
-def _parse(srclines, anon_name):
+def _parse(srclines, anon_name, **srckwargs):
     from ._global import parse_globals
 
-    source = _iter_source(srclines)
-    #source = _iter_source(srclines, showtext=True)
+    source = _iter_source(srclines, **srckwargs)
     for result in parse_globals(source, anon_name):
         # XXX Handle blocks here instead of in parse_globals().
         yield result
 
 
-def _iter_source(lines, *, maxtext=20_000, maxlines=700, showtext=False):
+# We use defaults that cover most files.  Files with bigger declarations
+# are covered elsewhere (MAX_SIZES in cpython/_parser.py).
+
+def _iter_source(lines, *, maxtext=11_000, maxlines=200, showtext=False):
     maxtext = maxtext if maxtext and maxtext > 0 else None
     maxlines = maxlines if maxlines and maxlines > 0 else None
     filestack = []
